@@ -87,8 +87,8 @@ static int data_handler(void *user, const char *section, const char *name,
 {
     UNUSED(user);
 
-    // If section matches user input or unnamed section
-    if ((!strcmp(section, c->section_name)) || (!strcmp(section, "")))
+    // If section matches current section being parsed
+    if (!strcmp(section, c->section_name))
     {
         // Set configuration parameter if found
         if (!strcmp(name, "tty"))
@@ -124,25 +124,79 @@ static int data_handler(void *user, const char *section, const char *name,
         }
         else if (!strcmp(name, "no-autoconnect"))
         {
-            option.no_autoconnect = atoi(value);
+            if (!strcmp(value, "enable"))
+            {
+                option.no_autoconnect = true;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.no_autoconnect = false;
+            }
         }
         else if (!strcmp(name, "log"))
         {
-            option.log = atoi(value);
+            if (!strcmp(value, "enable"))
+            {
+                option.log = true;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.log = false;
+            }
         }
-        else if (!strcmp(name, "local-echo"))
-        {
-            option.local_echo = atoi(value);
-        }
-        else if (!strcmp(name, "timestamp"))
-        {
-            option.timestamp = timestamp_option_parse(value);
-        }
-        else if (!strcmp(name, "log-filename"))
+        else if (!strcmp(name, "log-file"))
         {
             asprintf(&c->log_filename, "%s", value);
             option.log_filename = c->log_filename;
         }
+        else if (!strcmp(name, "log-strip"))
+        {
+            if (!strcmp(value, "enable"))
+            {
+                option.log_strip = true;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.log_strip = false;
+            }
+        }
+        else if (!strcmp(name, "local-echo"))
+        {
+            if (!strcmp(value, "enable"))
+            {
+                option.local_echo = true;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.local_echo = false;
+            }
+        }
+        else if (!strcmp(name, "hex-mode"))
+        {
+            if (!strcmp(value, "enable"))
+            {
+                option.hex_mode = true;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.hex_mode = false;
+            }
+        }
+        else if (!strcmp(name, "timestamp"))
+        {
+            if (!strcmp(value, "enable"))
+            {
+                option.timestamp = TIMESTAMP_24HOUR;
+            }
+            else if (!strcmp(value, "disable"))
+            {
+                option.timestamp = TIMESTAMP_NONE;
+            }
+        }
+        else if (!strcmp(name, "timestamp-format"))
+        {
+            option.timestamp = timestamp_option_parse(value);
+        }
         else if (!strcmp(name, "map"))
         {
             asprintf(&c->map, "%s", value);
@@ -150,7 +204,23 @@ static int data_handler(void *user, const char *section, const char *name,
         }
         else if (!strcmp(name, "color"))
         {
+            if (!strcmp(value, "list"))
+            {
+                // Ignore
+                return 0;
+            }
+
+            if (!strcmp(value, "none"))
+            {
+                option.color = -1; // No color
+                return 0;
+            }
+
             option.color = atoi(value);
+            if ((option.color < 0) || (option.color > 255))
+            {
+                option.color = -1; // No color
+            }
         }
         else if (!strcmp(name, "socket"))
         {
@@ -220,25 +290,32 @@ static int resolve_config_file(void)
         return 0;
     }
 
+    free(c->path);
+
     asprintf(&c->path, "%s/.config/tio/tiorc", getenv("HOME"));
     if (!access(c->path, F_OK))
     {
         return 0;
     }
 
+    free(c->path);
+
     asprintf(&c->path, "%s/.tiorc", getenv("HOME"));
     if (!access(c->path, F_OK))
     {
         return 0;
     }
 
+    free(c->path);
+
+    c->path = NULL;
+
     return -EINVAL;
 }
 
-void config_file_parse(const int argc, char *argv[])
+void config_file_parse(void)
 {
     int ret;
-    int i;
 
     c = malloc(sizeof(struct config_t));
     memset(c, 0, sizeof(struct config_t));
@@ -250,14 +327,8 @@ void config_file_parse(const int argc, char *argv[])
         return;
     }
 
-    for (i = 1; i < argc; i++)
-    {
-        if (argv[i][0] != '-')
-        {
-            c->user = argv[i];
-            break;
-        }
-    }
+    // Set user input which may be tty device or sub config
+    c->user = option.tty_device;
 
     if (!c->user)
     {
@@ -272,6 +343,7 @@ void config_file_parse(const int argc, char *argv[])
         fprintf(stderr, "Error: Unable to parse configuration file (%d)", ret);
         exit(EXIT_FAILURE);
     }
+    free(c->section_name);
     c->section_name = NULL;
 
     // Find matching section
@@ -286,13 +358,15 @@ void config_file_parse(const int argc, char *argv[])
         }
     }
 
-    // Parse settings of found section
+    // Parse settings of found section (sub config)
     ret = ini_parse(c->path, data_handler, NULL);
     if (ret < 0)
     {
         fprintf(stderr, "Error: Unable to parse configuration file (%d)", ret);
         exit(EXIT_FAILURE);
     }
+
+    atexit(&config_exit);
 }
 
 void config_exit(void)
@@ -310,14 +384,14 @@ void config_exit(void)
     free(c);
 }
 
-void config_file_print()
+void config_file_print(void)
 {
     if (c->path != NULL)
     {
         tio_printf(" Path: %s", c->path);
         if (c->section_name != NULL)
         {
-            tio_printf(" Active config section: %s", c->section_name);
+            tio_printf(" Active sub-configuration: %s", c->section_name);
         }
     }
 }
