@@ -34,6 +34,15 @@
 #include "error.h"
 #include "misc.h"
 #include "print.h"
+#include "tty.h"
+
+enum opt_t
+{
+    OPT_NONE,
+    OPT_TIMESTAMP_FORMAT,
+    OPT_LOG_FILE,
+    OPT_LOG_STRIP,
+};
 
 /* Default options */
 struct option_t option =
@@ -47,13 +56,14 @@ struct option_t option =
     .output_delay = 0,
     .no_autoconnect = false,
     .log = false,
+    .log_filename = NULL,
+    .log_strip = false,
     .local_echo = false,
     .timestamp = TIMESTAMP_NONE,
-    .list_devices = false,
-    .log_filename = NULL,
     .socket = NULL,
     .map = "",
-    .color = -1,
+    .color = 15,
+    .hex_mode = false,
 };
 
 void print_help(char *argv[])
@@ -61,30 +71,32 @@ void print_help(char *argv[])
     printf("Usage: %s [<options>] <tty-device|config>\n", argv[0]);
     printf("\n");
     printf("Options:\n");
-    printf("  -b, --baudrate <bps>        Baud rate (default: 115200)\n");
-    printf("  -d, --databits 5|6|7|8      Data bits (default: 8)\n");
-    printf("  -f, --flow hard|soft|none   Flow control (default: none)\n");
-    printf("  -s, --stopbits 1|2          Stop bits (default: 1)\n");
-    printf("  -p, --parity odd|even|none  Parity (default: none)\n");
-    printf("  -o, --output-delay <ms>     Output delay (default: 0)\n");
-    printf("  -n, --no-autoconnect        Disable automatic connect\n");
-    printf("  -e, --local-echo            Enable local echo\n");
-    printf("  -t, --timestamp[=<format>]  Enable timestamp (default: 24hour)\n");
-    printf("  -L, --list-devices          List available serial devices\n");
-    printf("  -l, --log[=<filename>]      Log to file\n");
-    printf("  -m, --map <flags>           Map special characters\n");
-    printf("  -c, --color <code>          Colorize tio text\n");
-    printf("  -S, --socket <socket>       Listen on socket\n");
-    printf("  -v, --version               Display version\n");
-    printf("  -h, --help                  Display help\n");
+    printf("  -b, --baudrate <bps>             Baud rate (default: 115200)\n");
+    printf("  -d, --databits 5|6|7|8           Data bits (default: 8)\n");
+    printf("  -f, --flow hard|soft|none        Flow control (default: none)\n");
+    printf("  -s, --stopbits 1|2               Stop bits (default: 1)\n");
+    printf("  -p, --parity odd|even|none       Parity (default: none)\n");
+    printf("  -o, --output-delay <ms>          Output delay (default: 0)\n");
+    printf("  -n, --no-autoconnect             Disable automatic connect\n");
+    printf("  -e, --local-echo                 Enable local echo\n");
+    printf("  -t, --timestamp                  Enable line timestamp\n");
+    printf("      --timestamp-format <format>  Set timestamp format (default: 24hour)\n");
+    printf("  -L, --list-devices               List available serial devices\n");
+    printf("  -l, --log                        Enable log to file\n");
+    printf("      --log-file <filename>        Set log filename\n");
+    printf("      --log-strip                  Strip control characters and escape sequences\n");
+    printf("  -m, --map <flags>                Map special characters\n");
+    printf("  -c, --color 0..255|none|list     Colorize tio text (default: 15)\n");
+    printf("  -S, --socket <socket>            Redirect I/O to socket\n");
+    printf("  -x, --hex-mode                   Enable hexadecimal mode\n");
+    printf("  -v, --version                    Display version\n");
+    printf("  -h, --help                       Display help\n");
     printf("\n");
     printf("Options may be set via configuration file.\n");
     printf("\n");
     printf("In session, press ctrl-t q to quit.\n");
     printf("\n");
     printf("See the man page for more details.\n");
-    printf("\n");
-
 }
 
 const char* timestamp_token(enum timestamp_t timestamp)
@@ -129,7 +141,7 @@ enum timestamp_t timestamp_option_parse(const char *arg)
         }
         else
         {
-            printf("Warning: Unknown timestamp type, falling back to '24hour' default format\n");
+            warning_printf("Unknown timestamp type, falling back to '24hour' default format");
         }
     }
 
@@ -170,30 +182,34 @@ void options_parse(int argc, char *argv[])
     {
         static struct option long_options[] =
         {
-            {"baudrate",       required_argument, 0, 'b'},
-            {"databits",       required_argument, 0, 'd'},
-            {"flow",           required_argument, 0, 'f'},
-            {"stopbits",       required_argument, 0, 's'},
-            {"parity",         required_argument, 0, 'p'},
-            {"output-delay",   required_argument, 0, 'o'},
-            {"no-autoconnect", no_argument,       0, 'n'},
-            {"local-echo",     no_argument,       0, 'e'},
-            {"timestamp",      optional_argument, 0, 't'},
-            {"list-devices",   no_argument,       0, 'L'},
-            {"log",            optional_argument, 0, 'l'},
-            {"socket",         required_argument, 0, 'S'},
-            {"map",            required_argument, 0, 'm'},
-            {"color",          required_argument, 0, 'c'},
-            {"version",        no_argument,       0, 'v'},
-            {"help",           no_argument,       0, 'h'},
-            {0,                0,                 0,  0 }
+            {"baudrate",         required_argument, 0, 'b'                  },
+            {"databits",         required_argument, 0, 'd'                  },
+            {"flow",             required_argument, 0, 'f'                  },
+            {"stopbits",         required_argument, 0, 's'                  },
+            {"parity",           required_argument, 0, 'p'                  },
+            {"output-delay",     required_argument, 0, 'o'                  },
+            {"no-autoconnect",   no_argument,       0, 'n'                  },
+            {"local-echo",       no_argument,       0, 'e'                  },
+            {"timestamp",        no_argument,       0, 't'                  },
+            {"timestamp-format", required_argument, 0, OPT_TIMESTAMP_FORMAT },
+            {"list-devices",     no_argument,       0, 'L'                  },
+            {"log",              no_argument,       0, 'l'                  },
+            {"log-file",         required_argument, 0, OPT_LOG_FILE         },
+            {"log-strip",        no_argument,       0, OPT_LOG_STRIP        },
+            {"socket",           required_argument, 0, 'S'                  },
+            {"map",              required_argument, 0, 'm'                  },
+            {"color",            required_argument, 0, 'c'                  },
+            {"hex-mode",         no_argument,       0, 'x'                  },
+            {"version",          no_argument,       0, 'v'                  },
+            {"help",             no_argument,       0, 'h'                  },
+            {0,                  0,                 0,  0                   }
         };
 
         /* getopt_long stores the option index here */
         int option_index = 0;
 
         /* Parse argument using getopt_long */
-        c = getopt_long(argc, argv, "b:d:f:s:p:o:net::Ll::S:m:c:vh", long_options, &option_index);
+        c = getopt_long(argc, argv, "b:d:f:s:p:o:netLlS:m:c:xvh", long_options, &option_index);
 
         /* Detect the end of the options */
         if (c == -1)
@@ -244,18 +260,30 @@ void options_parse(int argc, char *argv[])
                 break;
 
             case 't':
+                option.timestamp = TIMESTAMP_24HOUR;
+                break;
+
+            case OPT_TIMESTAMP_FORMAT:
                 option.timestamp = timestamp_option_parse(optarg);
                 break;
 
             case 'L':
-                option.list_devices = true;
+                list_serial_devices();
+                exit(EXIT_SUCCESS);
                 break;
 
             case 'l':
                 option.log = true;
+                break;
+
+            case OPT_LOG_FILE:
                 option.log_filename = optarg;
                 break;
 
+            case OPT_LOG_STRIP:
+                option.log_strip = true;
+                break;
+
             case 'S':
                 option.socket = optarg;
                 break;
@@ -265,13 +293,7 @@ void options_parse(int argc, char *argv[])
                 break;
 
             case 'c':
-                option.color = string_to_long(optarg);
-                if (option.color > 255)
-                {
-                    printf("Error: Invalid color code\n");
-                    exit(EXIT_FAILURE);
-                }
-                if (option.color < 0)
+                if (!strcmp(optarg, "list"))
                 {
                     // Print available color codes
                     printf("Available color codes:\n");
@@ -281,6 +303,23 @@ void options_parse(int argc, char *argv[])
                     }
                     exit(EXIT_SUCCESS);
                 }
+
+                if (!strcmp(optarg, "none"))
+                {
+                    option.color = -1;
+                    break;
+                }
+
+                option.color = string_to_long(optarg);
+                if ((option.color < 0) || (option.color > 255))
+                {
+                    printf("Error: Invalid color code\n");
+                    exit(EXIT_FAILURE);
+                }
+                break;
+
+            case 'x':
+                option.hex_mode = true;
                 break;
 
             case 'v':
@@ -308,11 +347,6 @@ void options_parse(int argc, char *argv[])
         }
     }
 
-    if (option.list_devices)
-    {
-        return;
-    }
-
     /* Assume first non-option is the tty device name */
     if (strcmp(option.tty_device, ""))
 	    optind++;
@@ -321,7 +355,7 @@ void options_parse(int argc, char *argv[])
 
     if (strlen(option.tty_device) == 0)
     {
-        printf("Error: Missing device name\n");
+        printf("Error: Missing device or config name\n");
         exit(EXIT_FAILURE);
     }
 
@@ -335,3 +369,16 @@ void options_parse(int argc, char *argv[])
         exit(EXIT_FAILURE);
     }
 }
+
+void options_parse_final(int argc, char *argv[])
+{
+    /* Preserve tty device which may have been set by configuration file */
+    const char *tty_device = option.tty_device;
+
+    /* Do 2nd pass to override settings set by configuration file */
+    optind = 1; // Reset option index to restart scanning of argv
+    options_parse(argc, argv);
+
+    /* Restore tty device */
+    option.tty_device = tty_device;
+}
