@@ -32,7 +32,9 @@ enum timestamp_t
     TIMESTAMP_24HOUR,
     TIMESTAMP_24HOUR_START,
     TIMESTAMP_ISO8601,
+    TIMESTAMP_END,
 };
+
 const char* timestamp_token(enum timestamp_t timestamp);
 enum timestamp_t timestamp_option_parse(const char *arg);
 
@@ -48,16 +50,18 @@ struct option_t
     int output_delay;
     bool no_autoconnect;
     bool log;
+    bool log_strip;
     bool local_echo;
     enum timestamp_t timestamp;
-    bool list_devices;
     const char *log_filename;
     const char *map;
     const char *socket;
     int color;
+    bool hex_mode;
 };
 
 extern struct option_t option;
 
 void options_print();
 void options_parse(int argc, char *argv[]);
+void options_parse_final(int argc, char *argv[]);
