@@ -63,6 +63,8 @@ extern int iossiospeed(int fd, int baudrate);
 #define PATH_SERIAL_DEVICES "/dev/serial/by-id/"
 #endif
 
+bool interactive_mode = true;
+
 static struct termios tio, tio_old, stdout_new, stdout_old, stdin_new, stdin_old;
 static unsigned long rx_total = 0, tx_total = 0;
 static bool connected = false;
@@ -74,7 +76,60 @@ static bool map_i_nl_crnl = false;
 static bool map_o_cr_nl = false;
 static bool map_o_nl_crnl = false;
 static bool map_o_del_bs = false;
+static char hex_chars[2];
+static unsigned char hex_char_index = 0;
 
+static void optional_local_echo(char c)
+{
+    if (!option.local_echo)
+        return;
+    print(c);
+    fflush(stdout);
+    if (option.log)
+        log_write(c);
+}
+
+inline static bool is_valid_hex(char c)
+{
+    return ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F'));
+}
+
+inline static unsigned char char_to_nibble(char c)
+{
+    if(c >= '0' && c <= '9'){
+        return c - '0';
+    } else if(c >= 'a' && c <= 'f'){
+        return c - 'a' + 10;
+    } else if(c >= 'A' && c <= 'F'){
+        return c - 'A' + 10;
+    } else {
+        return 0;
+    }
+}
+
+static void output_hex(char c)
+{
+    hex_chars[hex_char_index++] = c;
+
+    if (hex_char_index == 2)
+    {
+        unsigned char hex_value = char_to_nibble(hex_chars[0]) << 4 | (char_to_nibble(hex_chars[1]) & 0x0F);
+        hex_char_index = 0;
+
+        optional_local_echo(hex_value);
+
+        ssize_t status = write(fd, &hex_value, 1);
+        if (status < 0)
+        {
+            warning_printf("Could not write to tty device");
+        } else
+        {
+            tx_total++;
+        }
+
+        fsync(fd);
+    }
+}
 
 static void toggle_line(const char *line_name, int mask)
 {
@@ -82,7 +137,7 @@ static void toggle_line(const char *line_name, int mask)
 
     if (ioctl(fd, TIOCMGET, &state) < 0)
     {
-        error_printf("Could not get line state: %s", strerror(errno));
+        warning_printf("Could not get line state (%s)", strerror(errno));
     }
     else
     {
@@ -97,7 +152,7 @@ static void toggle_line(const char *line_name, int mask)
             tio_printf("set %s to HIGH", line_name);
         }
         if (ioctl(fd, TIOCMSET, &state) < 0)
-            error_printf("Could not set line state: %s", strerror(errno));
+            warning_printf("Could not set line state (%s)", strerror(errno));
     }
 }
 
@@ -143,7 +198,7 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
             case KEY_SHIFT_L:
                 if (ioctl(fd, TIOCMGET, &state) < 0)
                 {
-                    error_printf("Could not get line state: %s", strerror(errno));
+                    warning_printf("Could not get line state (%s)", strerror(errno));
                     break;
                 }
                 tio_printf("Line states:");
@@ -174,6 +229,7 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
 
             case KEY_E:
                 option.local_echo = !option.local_echo;
+                tio_printf("Switched local echo %s", option.local_echo ? "on" : "off");
                 break;
 
             case KEY_H:
@@ -216,7 +272,25 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 break;
 
             case KEY_SHIFT_T:
-                option.timestamp = !option.timestamp;
+                option.timestamp += 1;
+                switch (option.timestamp)
+                {
+                    case TIMESTAMP_NONE:
+                        break;
+                    case TIMESTAMP_24HOUR:
+                        tio_printf("Switched to 24hour timestamp mode");
+                        break;
+                    case TIMESTAMP_24HOUR_START:
+                        tio_printf("Switched to 24hour-start timestamp mode");
+                        break;
+                    case TIMESTAMP_ISO8601:
+                        tio_printf("Switched to iso8601 timestamp mode");
+                        break;
+                    case TIMESTAMP_END:
+                        option.timestamp = TIMESTAMP_NONE;
+                        tio_printf("Switched timestamp off");
+                        break;
+                }
                 break;
 
             case KEY_V:
@@ -329,8 +403,8 @@ void tty_configure(void)
     /* Set speed */
     switch (option.baudrate)
     {
-        /* The macro below expands into switch cases autogenerated by the
-         * configure script. Each switch case verifies and configures the baud
+        /* The macro below expands into switch cases autogenerated by meson
+         * configure. Each switch case verifies and configures the baud
          * rate and is of the form:
          *
          * case $baudrate: baudrate = B$baudrate; break;
@@ -585,16 +659,6 @@ void tty_restore(void)
         tty_disconnect();
 }
 
-static void optional_local_echo(char c)
-{
-    if (!option.local_echo)
-        return;
-    print(c);
-    fflush(stdout);
-    if (option.log)
-        log_write(c);
-}
-
 int tty_connect(void)
 {
     fd_set rdfs;           /* Read file descriptor set */
@@ -644,6 +708,18 @@ int tty_connect(void)
     if (option.timestamp)
         next_timestamp = true;
 
+    /* Manage print output mode */
+    if (option.hex_mode)
+    {
+        print = print_hex;
+        print_mode = HEX;
+    }
+    else
+    {
+        print = print_normal;
+        print_mode = NORMAL;
+    }
+
     /* Save current port settings */
     if (tcgetattr(fd, &tio_old) < 0)
         goto error_tcgetattr;
@@ -781,17 +857,21 @@ int tty_connect(void)
                     goto error_read;
                 }
 
-                /* Forward input to output except ctrl-t key */
+                /* Forward input to output */
                 output_char = input_char;
-                if (input_char == KEY_CTRL_T)
-                    forward = false;
 
-                /* Handle commands */
-                handle_command_sequence(input_char, previous_char, &output_char, &forward);
+                if (interactive_mode)
+                {
+                    /* Do not forward ctrl-t key */
+                    if (input_char == KEY_CTRL_T)
+                        forward = false;
 
-                /* Save previous key */
-                previous_char = input_char;
+                    /* Handle commands */
+                    handle_command_sequence(input_char, previous_char, &output_char, &forward);
 
+                    /* Save previous key */
+                    previous_char = input_char;
+                }
             }
             else
             {
@@ -800,6 +880,15 @@ int tty_connect(void)
 
             if (forward)
             {
+                if (print_mode == HEX)
+                {
+                    if (!is_valid_hex(input_char))
+                    {
+                        warning_printf("Invalid hex character: '%d' (0x%02x)", input_char, input_char);
+                        continue;
+                    }
+                }
+
                 /* Map output character */
                 if ((output_char == 127) && (map_o_del_bs))
                     output_char = '\b';
@@ -820,15 +909,22 @@ int tty_connect(void)
                     delay(option.output_delay);
                 } else
                 {
-                    /* Send output to tty device */
-                    optional_local_echo(output_char);
-                    status = write(fd, &output_char, 1);
-                    if (status < 0)
-                        warning_printf("Could not write to tty device");
-                    fsync(fd);
+                    if (print_mode == HEX)
+                    {
+                        output_hex(output_char);
+                    }
+                    else
+                    {
+                        /* Send output to tty device */
+                        optional_local_echo(output_char);
+                        status = write(fd, &output_char, 1);
+                        if (status < 0)
+                            warning_printf("Could not write to tty device");
+                        fsync(fd);
 
-                    /* Update transmit statistics */
-                    tx_total++;
+                        /* Update transmit statistics */
+                        tx_total++;
+                    }
 
                     /* Insert output delay */
                     delay(option.output_delay);
